@@ -12,7 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone import assignment as keystone_assignment
+from keystone.assignment.backends import base
 from keystone.common import sql
 from keystone import exception
 from keystone.i18n import _
@@ -40,7 +40,7 @@ class AssignmentType(object):
         raise exception.AssignmentTypeCalculationError(**locals())
 
 
-class Assignment(keystone_assignment.AssignmentDriverV9):
+class Assignment(base.AssignmentDriverV9):
 
     def default_role_driver(self):
         return 'sql'
@@ -151,7 +151,7 @@ class Assignment(keystone_assignment.AssignmentDriverV9):
         return [AssignmentType.USER_DOMAIN, AssignmentType.GROUP_DOMAIN]
 
     def _get_assignment_types(self, user, group, project, domain):
-        """Returns a list of role assignment types based on provided entities
+        """Return a list of role assignment types based on provided entities.
 
         If one of user or group (the "actor") as well as one of project or
         domain (the "target") are provided, the list will contain the role
