@@ -20,14 +20,14 @@ import time
 from dogpile.cache import api
 from dogpile.cache.backends import memcached
 from oslo_cache.backends import memcache_pool
-from oslo_config import cfg
 from six.moves import range
 
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 NO_VALUE = api.NO_VALUE
 random = _random.SystemRandom()
 
