@@ -15,31 +15,22 @@
 #    under the License.
 
 import os
-import sys
 
 import migrate
 from migrate import exceptions
-from oslo_config import cfg
 from oslo_db.sqlalchemy import migration
-from oslo_utils import importutils
 import six
 import sqlalchemy
 
 from keystone.common import sql
-from keystone import contrib
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 
 
-CONF = cfg.CONF
-DEFAULT_EXTENSIONS = []
+CONF = keystone.conf.CONF
 
-MIGRATED_EXTENSIONS = ['endpoint_policy',
-                       'federation',
-                       'oauth1',
-                       'revoke',
-                       'endpoint_filter'
-                       ]
+USE_TRIGGERS = True
 
 
 #  Different RDBMSs use different schemes for naming the Foreign Key
@@ -95,7 +86,7 @@ def add_constraints(constraints):
 
 
 def rename_tables_with_constraints(renames, constraints, engine):
-    """Renames tables with foreign key constraints.
+    """Rename tables with foreign key constraints.
 
     Tables are renamed after first removing constraints. The constraints are
     replaced after the rename is complete.
@@ -136,8 +127,25 @@ def _sync_common_repo(version):
                           init_version=init_version, sanity_check=False)
 
 
+def _sync_repo(repo_name):
+    abs_path = find_migrate_repo(repo_name=repo_name)
+    with sql.session_for_write() as session:
+        engine = session.get_bind()
+        # Register the repo with the version control API
+        # If it already knows about the repo, it will throw
+        # an exception that we can safely ignore
+        try:
+            migration.db_version_control(engine, abs_path)
+        except (migration.exception.DbMigrationError,
+                exceptions.DatabaseAlreadyControlledError):  # nosec
+            pass
+        init_version = get_init_version(abs_path=abs_path)
+        migration.db_sync(engine, abs_path,
+                          init_version=init_version, sanity_check=False)
+
+
 def get_init_version(abs_path=None):
-    """Get the initial version of a migrate repository
+    """Get the initial version of a migrate repository.
 
     :param abs_path: Absolute path to migrate repository.
     :return:         initial version number or None, if DB is empty.
@@ -158,10 +166,10 @@ def get_init_version(abs_path=None):
     return oldest - 1
 
 
-def _assert_not_schema_downgrade(extension=None, version=None):
+def _assert_not_schema_downgrade(version=None):
     if version is not None:
         try:
-            current_ver = int(six.text_type(get_db_version(extension)))
+            current_ver = int(six.text_type(get_db_version()))
             if int(version) < current_ver:
                 raise migration.exception.DbMigrationError(
                     _("Unable to downgrade schema"))
@@ -171,75 +179,76 @@ def _assert_not_schema_downgrade(extension=None, version=None):
             pass
 
 
-def _sync_extension_repo(extension, version):
-    if extension in MIGRATED_EXTENSIONS:
-        raise exception.MigrationMovedFailure(extension=extension)
+def offline_sync_database_to_version(version=None):
+    """Perform and off-line sync of the database.
 
-    with sql.session_for_write() as session:
-        engine = session.get_bind()
+    Migrate the database up to the latest version, doing the equivalent of
+    the cycle of --expand, --migrate and --contract, for when an offline
+    upgrade is being performed.
 
-        try:
-            package_name = '.'.join((contrib.__name__, extension))
-            package = importutils.import_module(package_name)
-        except ImportError:
-            raise ImportError(_("%s extension does not exist.")
-                              % package_name)
-        try:
-            abs_path = find_migrate_repo(package)
-            try:
-                migration.db_version_control(engine, abs_path)
-            # Register the repo with the version control API
-            # If it already knows about the repo, it will throw
-            # an exception that we can safely ignore
-            except exceptions.DatabaseAlreadyControlledError:  # nosec
-                pass
-        except exception.MigrationNotProvided as e:
-            print(e)
-            sys.exit(1)
+    If a version is specified then only migrate the database up to that
+    version. Downgrading is not supported. If version is specified, then only
+    the main database migration is carried out - and the expand, migration and
+    contract phases will NOT be run.
 
-        _assert_not_schema_downgrade(extension=extension, version=version)
+    """
+    global USE_TRIGGERS
 
-        init_version = get_init_version(abs_path=abs_path)
+    # This flags let's us bypass trigger setup & teardown for non-rolling
+    # upgrades. We set this as a global variable immediately before handing off
+    # to sqlalchemy-migrate, because we can't pass arguments directly to
+    # migrations that depend on it. We could also register this as a CONF
+    # option, but the idea here is that we aren't exposing a new API.
+    USE_TRIGGERS = False
 
-        migration.db_sync(engine, abs_path, version=version,
-                          init_version=init_version, sanity_check=False)
-
-
-def sync_database_to_version(extension=None, version=None):
-    if not extension:
+    if version:
         _sync_common_repo(version)
-        # If version is greater than 0, it is for the common
-        # repository only, and only that will be synchronized.
-        if version is None:
-            for default_extension in DEFAULT_EXTENSIONS:
-                _sync_extension_repo(default_extension, version)
     else:
-        _sync_extension_repo(extension, version)
+        expand_schema()
+        migrate_data()
+        contract_schema()
 
 
-def get_db_version(extension=None):
-    if not extension:
-        with sql.session_for_write() as session:
-            return migration.db_version(session.get_bind(),
-                                        find_migrate_repo(),
-                                        get_init_version())
-
-    try:
-        package_name = '.'.join((contrib.__name__, extension))
-        package = importutils.import_module(package_name)
-    except ImportError:
-        raise ImportError(_("%s extension does not exist.")
-                          % package_name)
-
+def get_db_version():
     with sql.session_for_write() as session:
-        return migration.db_version(
-            session.get_bind(), find_migrate_repo(package), 0)
+        return migration.db_version(session.get_bind(),
+                                    find_migrate_repo(),
+                                    get_init_version())
 
 
-def print_db_version(extension=None):
-    try:
-        db_version = get_db_version(extension=extension)
-        print(db_version)
-    except exception.MigrationNotProvided as e:
-        print(e)
-        sys.exit(1)
+def print_db_version():
+    print(get_db_version())
+
+
+def expand_schema():
+    """Expand the database schema ahead of data migration.
+
+    This is run manually by the keystone-manage command before the first
+    keystone node is migrated to the latest release.
+
+    """
+    # Make sure all the legacy migrations are run before we run any new
+    # expand migrations.
+    _sync_common_repo(version=None)
+    _sync_repo(repo_name='expand_repo')
+
+
+def migrate_data():
+    """Migrate data to match the new schema.
+
+    This is run manually by the keystone-manage command once the keystone
+    schema has been expanded for the new release.
+
+    """
+    _sync_repo(repo_name='data_migration_repo')
+
+
+def contract_schema():
+    """Contract the database.
+
+    This is run manually by the keystone-manage command once the keystone
+    nodes have been upgraded to the latest release and will remove any old
+    tables/columns that are no longer required.
+
+    """
+    _sync_repo(repo_name='contract_repo')
