@@ -14,7 +14,6 @@
 
 """Main entry point into the Identity service."""
 
-import abc
 import functools
 import os
 import threading
@@ -23,27 +22,34 @@ import uuid
 from oslo_config import cfg
 from oslo_log import log
 from oslo_log import versionutils
-import six
 
 from keystone import assignment  # TODO(lbragstad): Decouple this dependency
 from keystone.common import cache
 from keystone.common import clean
-from keystone.common import config
 from keystone.common import dependency
 from keystone.common import driver_hints
 from keystone.common import manager
+from keystone.common.validation import validators
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _, _LW
+from keystone.identity.backends import base as identity_interface
+from keystone.identity.mapping_backends import base as mapping_interface
 from keystone.identity.mapping_backends import mapping
+from keystone.identity.shadow_backends import base as shadow_interface
 from keystone import notifications
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 LOG = log.getLogger(__name__)
 
 MEMOIZE = cache.get_memoization_decorator(group='identity')
 
+ID_MAPPING_REGION = cache.create_region(name='id mapping')
+MEMOIZE_ID_MAPPING = cache.get_memoization_decorator(group='identity',
+                                                     region=ID_MAPPING_REGION)
+
 DOMAIN_CONF_FHEAD = 'keystone.'
 DOMAIN_CONF_FTAIL = '.conf'
 
@@ -56,29 +62,6 @@ REGISTRATION_ATTEMPTS = 10
 SQL_DRIVER = 'SQL'
 
 
-def filter_user(user_ref):
-    """Filter out private items in a user dict.
-
-    'password', 'tenants' and 'groups' are never returned.
-
-    :returns: user_ref
-
-    """
-    if user_ref:
-        user_ref = user_ref.copy()
-        user_ref.pop('password', None)
-        user_ref.pop('tenants', None)
-        user_ref.pop('groups', None)
-        user_ref.pop('domains', None)
-        try:
-            user_ref['extra'].pop('password', None)
-            user_ref['extra'].pop('tenants', None)
-        except KeyError:  # nosec
-            # ok to not have extra in the user_ref.
-            pass
-    return user_ref
-
-
 @dependency.requires('domain_config_api', 'resource_api')
 class DomainConfigs(dict):
     """Discover, store and provide access to domain specific configs.
@@ -139,7 +122,7 @@ class DomainConfigs(dict):
         # config dict to make sure we call the right driver
         domain_config = {}
         domain_config['cfg'] = cfg.ConfigOpts()
-        config.configure(conf=domain_config['cfg'])
+        keystone.conf.configure(conf=domain_config['cfg'])
         domain_config['cfg'](args=[], project='keystone',
                              default_config_files=file_list)
         domain_config['driver'] = self._load_driver(domain_config)
@@ -187,7 +170,7 @@ class DomainConfigs(dict):
     def _load_config_from_database(self, domain_id, specific_config):
 
         def _assert_no_more_than_one_sql_driver(domain_id, new_config):
-            """Ensure adding driver doesn't push us over the limit of 1
+            """Ensure adding driver doesn't push us over the limit of 1.
 
             The checks we make in this method need to take into account that
             we may be in a multiple process configuration and ensure that
@@ -276,7 +259,7 @@ class DomainConfigs(dict):
 
         domain_config = {}
         domain_config['cfg'] = cfg.ConfigOpts()
-        config.configure(conf=domain_config['cfg'])
+        keystone.conf.configure(conf=domain_config['cfg'])
         domain_config['cfg'](args=[], project='keystone',
                              default_config_files=[])
 
@@ -409,7 +392,7 @@ class DomainConfigs(dict):
 
 
 def domains_configured(f):
-    """Wraps API calls to lazy load domain configs after init.
+    """Wrap API calls to lazy load domain configs after init.
 
     This is required since the assignment manager needs to be initialized
     before this manager, and yet this manager's init wants to be
@@ -435,7 +418,7 @@ def domains_configured(f):
 
 
 def exception_translated(exception_type):
-    """Wraps API calls to map to correct exception."""
+    """Wrap API calls to map to correct exception."""
     def _exception_translated(f):
         @functools.wraps(f)
         def wrapper(self, *args, **kwargs):
@@ -565,8 +548,8 @@ class Manager(manager.Manager):
           single domain/driver LDAP configurations were previously supported).
         - If the driver does support UUIDs, then we always create a mapping
           entry, but use the local UUID as the public ID.  The exception to
-        - this is that if we just have single driver (i.e. not using specific
-          multi-domain configs), then we don't both with the mapping at all.
+          this is that if we just have single driver (i.e. not using specific
+          multi-domain configs), then we don't bother with the mapping at all.
 
         """
         conf = CONF.identity
@@ -595,7 +578,7 @@ class Manager(manager.Manager):
             raise ValueError(_('Expected dict or list: %s') % type(ref))
 
     def _needs_post_processing(self, driver):
-        """Returns whether entity from driver needs domain added or mapping."""
+        """Return whether entity from driver needs domain added or mapping."""
         return (driver is not self.driver or not driver.generates_uuids() or
                 not driver.is_domain_aware())
 
@@ -627,7 +610,7 @@ class Manager(manager.Manager):
         return ref
 
     def _insert_domain_id_if_needed(self, ref, driver, domain_id, conf):
-        """Inserts the domain ID into the ref, if required.
+        """Insert the domain ID into the ref, if required.
 
         If the driver can't handle domains, then we need to insert the
         domain_id into the entity being returned.  If the domain_id is
@@ -640,7 +623,7 @@ class Manager(manager.Manager):
             ref['domain_id'] = domain_id
 
     def _is_mapping_needed(self, driver):
-        """Returns whether mapping is needed.
+        """Return whether mapping is needed.
 
         There are two situations where we must use the mapping:
         - this isn't the default driver (i.e. multiple backends), or
@@ -769,7 +752,7 @@ class Manager(manager.Manager):
 
     def _assert_user_and_group_in_same_backend(
             self, user_entity_id, user_driver, group_entity_id, group_driver):
-        """Ensures that user and group IDs are backed by the same backend.
+        """Ensure that user and group IDs are backed by the same backend.
 
         Raise a CrossBackendNotAllowed exception if they are not from the same
         backend, otherwise return None.
@@ -799,7 +782,7 @@ class Manager(manager.Manager):
             hints.add_filter('domain_id', domain_id)
 
     def _set_list_limit_in_hints(self, hints, driver):
-        """Set list limit in hints from driver
+        """Set list limit in hints from driver.
 
         If a hints list is provided, the wrapper will insert the relevant
         limit into the hints so that the underlying driver call can try and
@@ -843,23 +826,46 @@ class Manager(manager.Manager):
     @notifications.emit_event('authenticate')
     @domains_configured
     @exception_translated('assertion')
-    def authenticate(self, context, user_id, password):
+    def authenticate(self, request, user_id, password):
         domain_id, driver, entity_id = (
             self._get_domain_driver_and_entity_id(user_id))
         ref = driver.authenticate(entity_id, password)
-        return self._set_domain_id_and_mapping(
+        ref = self._set_domain_id_and_mapping(
             ref, domain_id, driver, mapping.EntityType.USER)
+        ref = self._shadow_nonlocal_user(ref)
+        self.shadow_users_api.set_last_active_at(ref['id'])
+        return ref
+
+    def _assert_default_project_id_is_not_domain(self, default_project_id):
+        if default_project_id:
+            # make sure project is not a domain
+            try:
+                project_ref = self.resource_api.get_project(default_project_id)
+                if project_ref['is_domain'] is True:
+                    msg = _("User's default project ID cannot be a "
+                            "domain ID: %s")
+                    raise exception.ValidationError(
+                        message=(msg % default_project_id))
+            except exception.ProjectNotFound:
+                # should be idempotent if project is not found so that it is
+                # backward compatible
+                pass
 
     @domains_configured
     @exception_translated('user')
     def create_user(self, user_ref, initiator=None):
         user = user_ref.copy()
+        if 'password' in user:
+            validators.validate_password(user['password'])
         user['name'] = clean.user_name(user['name'])
         user.setdefault('enabled', True)
         user['enabled'] = clean.user_enabled(user['enabled'])
         domain_id = user['domain_id']
         self.resource_api.get_domain(domain_id)
 
+        self._assert_default_project_id_is_not_domain(
+            user_ref.get('default_project_id'))
+
         # For creating a user, the domain is in the object itself
         domain_id = user_ref['domain_id']
         driver = self._select_identity_driver(domain_id)
@@ -934,6 +940,8 @@ class Manager(manager.Manager):
     def update_user(self, user_id, user_ref, initiator=None):
         old_user_ref = self.get_user(user_id)
         user = user_ref.copy()
+        if 'password' in user:
+            validators.validate_password(user['password'])
         if 'name' in user:
             user['name'] = clean.user_name(user['name'])
         if 'enabled' in user:
@@ -950,6 +958,9 @@ class Manager(manager.Manager):
             # public ID not the local ID
             user.pop('id')
 
+        self._assert_default_project_id_is_not_domain(
+            user_ref.get('default_project_id'))
+
         domain_id, driver, entity_id = (
             self._get_domain_driver_and_entity_id(user_id))
         user = self._clear_domain_id_if_domain_unaware(driver, user)
@@ -1005,6 +1016,7 @@ class Manager(manager.Manager):
         # the underlying driver so that it could conform to rules set down by
         # that particular driver type.
         group['id'] = uuid.uuid4().hex
+        group['name'] = clean.group_name(group['name'])
         ref = driver.create_group(group['id'], group)
 
         notifications.Audit.created(self._GROUP, group['id'], initiator)
@@ -1041,6 +1053,8 @@ class Manager(manager.Manager):
         domain_id, driver, entity_id = (
             self._get_domain_driver_and_entity_id(group_id))
         group = self._clear_domain_id_if_domain_unaware(driver, group)
+        if 'name' in group:
+            group['name'] = clean.group_name(group['name'])
         ref = driver.update_group(entity_id, group)
         self.get_group.invalidate(self, group_id)
         notifications.Audit.updated(self._GROUP, group_id, initiator)
@@ -1214,19 +1228,31 @@ class Manager(manager.Manager):
                                                 group_entity_id)
 
     @domains_configured
-    def change_password(self, context, user_id, original_password,
+    def change_password(self, request, user_id, original_password,
                         new_password):
 
         # authenticate() will raise an AssertionError if authentication fails
-        self.authenticate(context, user_id, original_password)
+        self.authenticate(request, user_id, original_password)
 
-        update_dict = {'password': new_password}
-        self.update_user(user_id, update_dict)
+        validators.validate_password(new_password)
+
+        domain_id, driver, entity_id = (
+            self._get_domain_driver_and_entity_id(user_id))
+        driver.change_password(entity_id, new_password)
+        notifications.Audit.updated(self._USER, user_id)
+        self.emit_invalidate_user_token_persistence(user_id)
+
+    @MEMOIZE
+    def _shadow_nonlocal_user(self, user):
+        try:
+            return self.shadow_users_api.get_user(user['id'])
+        except exception.UserNotFound:
+            return self.shadow_users_api.create_nonlocal_user(user)
 
     @MEMOIZE
     def shadow_federated_user(self, idp_id, protocol_id, unique_id,
                               display_name):
-        """Shadows a federated user by mapping to a user.
+        """Map a federated user to a user.
 
         :param idp_id: identity provider id
         :param protocol_id: protocol id
@@ -1250,241 +1276,19 @@ class Manager(manager.Manager):
             }
             user_dict = self.shadow_users_api.create_federated_user(
                 federated_dict)
+        self.shadow_users_api.set_last_active_at(user_dict['id'])
         return user_dict
 
 
-@six.add_metaclass(abc.ABCMeta)
-class IdentityDriverV8(object):
-    """Interface description for an Identity driver."""
-
-    def _get_conf(self):
-        try:
-            return self.conf or CONF
-        except AttributeError:
-            return CONF
-
-    def _get_list_limit(self):
-        conf = self._get_conf()
-        # use list_limit from domain-specific config. If list_limit in
-        # domain-specific config is not set, look it up in the default config
-        return (conf.identity.list_limit or conf.list_limit or
-                CONF.identity.list_limit or CONF.list_limit)
-
-    def is_domain_aware(self):
-        """Indicates if Driver supports domains."""
-        return True
-
-    def default_assignment_driver(self):
-        # TODO(morganfainberg): To be removed when assignment driver based
-        # upon [identity]/driver option is removed in the "O" release.
-        return 'sql'
-
-    @property
-    def is_sql(self):
-        """Indicates if this Driver uses SQL."""
-        return False
-
-    @property
-    def multiple_domains_supported(self):
-        return (self.is_domain_aware() or
-                CONF.identity.domain_specific_drivers_enabled)
-
-    def generates_uuids(self):
-        """Indicates if Driver generates UUIDs as the local entity ID."""
-        return True
-
-    @abc.abstractmethod
-    def authenticate(self, user_id, password):
-        """Authenticate a given user and password.
-
-        :returns: user_ref
-        :raises AssertionError: If user or password is invalid.
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    # user crud
-
-    @abc.abstractmethod
-    def create_user(self, user_id, user):
-        """Creates a new user.
-
-        :raises keystone.exception.Conflict: If a duplicate user exists.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def list_users(self, hints):
-        """List users in the system.
-
-        :param hints: filter hints which the driver should
-                      implement if at all possible.
-
-        :returns: a list of user_refs or an empty list.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def list_users_in_group(self, group_id, hints):
-        """List users in a group.
-
-        :param group_id: the group in question
-        :param hints: filter hints which the driver should
-                      implement if at all possible.
-
-        :returns: a list of user_refs or an empty list.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def get_user(self, user_id):
-        """Get a user by ID.
-
-        :returns: user_ref
-        :raises keystone.exception.UserNotFound: If the user doesn't exist.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def update_user(self, user_id, user):
-        """Updates an existing user.
-
-        :raises keystone.exception.UserNotFound: If the user doesn't exist.
-        :raises keystone.exception.Conflict: If a duplicate user exists.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def add_user_to_group(self, user_id, group_id):
-        """Adds a user to a group.
-
-        :raises keystone.exception.UserNotFound: If the user doesn't exist.
-        :raises keystone.exception.GroupNotFound: If the group doesn't exist.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def check_user_in_group(self, user_id, group_id):
-        """Checks if a user is a member of a group.
-
-        :raises keystone.exception.UserNotFound: If the user doesn't exist.
-        :raises keystone.exception.GroupNotFound: If the group doesn't exist.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def remove_user_from_group(self, user_id, group_id):
-        """Removes a user from a group.
-
-        :raises keystone.exception.NotFound: If the entity not found.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def delete_user(self, user_id):
-        """Deletes an existing user.
-
-        :raises keystone.exception.UserNotFound: If the user doesn't exist.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def get_user_by_name(self, user_name, domain_id):
-        """Get a user by name.
-
-        :returns: user_ref
-        :raises keystone.exception.UserNotFound: If the user doesn't exist.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    # group crud
-
-    @abc.abstractmethod
-    def create_group(self, group_id, group):
-        """Creates a new group.
-
-        :raises keystone.exception.Conflict: If a duplicate group exists.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def list_groups(self, hints):
-        """List groups in the system.
-
-        :param hints: filter hints which the driver should
-                      implement if at all possible.
-
-        :returns: a list of group_refs or an empty list.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def list_groups_for_user(self, user_id, hints):
-        """List groups a user is in
-
-        :param user_id: the user in question
-        :param hints: filter hints which the driver should
-                      implement if at all possible.
-
-        :returns: a list of group_refs or an empty list.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def get_group(self, group_id):
-        """Get a group by ID.
-
-        :returns: group_ref
-        :raises keystone.exception.GroupNotFound: If the group doesn't exist.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def get_group_by_name(self, group_name, domain_id):
-        """Get a group by name.
-
-        :returns: group_ref
-        :raises keystone.exception.GroupNotFound: If the group doesn't exist.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def update_group(self, group_id, group):
-        """Updates an existing group.
-
-        :raises keystone.exception.GroupNotFound: If the group doesn't exist.
-        :raises keystone.exception.Conflict: If a duplicate group exists.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def delete_group(self, group_id):
-        """Deletes an existing group.
-
-        :raises keystone.exception.GroupNotFound: If the group doesn't exist.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    # end of identity
-
-
-Driver = manager.create_legacy_driver(IdentityDriverV8)
+@versionutils.deprecated(
+    versionutils.deprecated.NEWTON,
+    what='keystone.identity.IdentityDriverV8',
+    in_favor_of='keystone.identity.backends.base.IdentityDriverV8',
+    remove_in=+1)
+class IdentityDriverV8(identity_interface.IdentityDriverV8):
+    pass
+
+Driver = manager.create_legacy_driver(identity_interface.IdentityDriverV8)
 
 
 @dependency.provider('id_mapping_api')
@@ -1496,70 +1300,59 @@ class MappingManager(manager.Manager):
     def __init__(self):
         super(MappingManager, self).__init__(CONF.identity_mapping.driver)
 
+    @MEMOIZE_ID_MAPPING
+    def _get_public_id(self, domain_id, local_id, entity_type):
+        return self.driver.get_public_id({'domain_id': domain_id,
+                                          'local_id': local_id,
+                                          'entity_type': entity_type})
 
-@six.add_metaclass(abc.ABCMeta)
-class MappingDriverV8(object):
-    """Interface description for an ID Mapping driver."""
-
-    @abc.abstractmethod
     def get_public_id(self, local_entity):
-        """Returns the public ID for the given local entity.
+        return self._get_public_id(local_entity['domain_id'],
+                                   local_entity['local_id'],
+                                   local_entity['entity_type'])
 
-        :param dict local_entity: Containing the entity domain, local ID and
-                                  type ('user' or 'group').
-        :returns: public ID, or None if no mapping is found.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
+    @MEMOIZE_ID_MAPPING
     def get_id_mapping(self, public_id):
-        """Returns the local mapping.
+        return self.driver.get_id_mapping(public_id)
 
-        :param public_id: The public ID for the mapping required.
-        :returns dict: Containing the entity domain, local ID and type. If no
-                       mapping is found, it returns None.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
     def create_id_mapping(self, local_entity, public_id=None):
-        """Create and store a mapping to a public_id.
+        public_id = self.driver.create_id_mapping(local_entity, public_id)
+        if MEMOIZE_ID_MAPPING.should_cache(public_id):
+            self._get_public_id.set(public_id, self,
+                                    local_entity['domain_id'],
+                                    local_entity['local_id'],
+                                    local_entity['entity_type'])
+            self.get_id_mapping.set(local_entity, self, public_id)
+        return public_id
 
-        :param dict local_entity: Containing the entity domain, local ID and
-                                  type ('user' or 'group').
-        :param public_id: If specified, this will be the public ID.  If this
-                          is not specified, a public ID will be generated.
-        :returns: public ID
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
     def delete_id_mapping(self, public_id):
-        """Deletes an entry for the given public_id.
+        local_entity = self.get_id_mapping.get(self, public_id)
+        self.driver.delete_id_mapping(public_id)
+        # Delete the key of entity from cache
+        if local_entity:
+            self._get_public_id.invalidate(self, local_entity['domain_id'],
+                                           local_entity['local_id'],
+                                           local_entity['entity_type'])
+        self.get_id_mapping.invalidate(self, public_id)
 
-        :param public_id: The public ID for the mapping to be deleted.
-
-        The method is silent if no mapping is found.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
     def purge_mappings(self, purge_filter):
-        """Purge selected identity mappings.
-
-        :param dict purge_filter: Containing the attributes of the filter that
-                                  defines which entries to purge. An empty
-                                  filter means purge all mappings.
-
-        """
-        raise exception.NotImplemented()  # pragma: no cover
+        # Purge mapping is rarely used and only used by the command client,
+        # it's quite complex to invalidate part of the cache based on the purge
+        # filters, so here invalidate the whole cache when purging mappings.
+        self.driver.purge_mappings(purge_filter)
+        ID_MAPPING_REGION.invalidate()
 
 
-MappingDriver = manager.create_legacy_driver(MappingDriverV8)
+@versionutils.deprecated(
+    versionutils.deprecated.NEWTON,
+    what='keystone.identity.MappingDriverV8',
+    in_favor_of='keystone.identity.mapping_backends.base.MappingDriverV8',
+    remove_in=+1)
+class MappingDriverV8(mapping_interface.MappingDriverV8):
+    pass
+
+
+MappingDriver = manager.create_legacy_driver(mapping_interface.MappingDriverV8)
 
 
 @dependency.provider('shadow_users_api')
@@ -1569,45 +1362,22 @@ class ShadowUsersManager(manager.Manager):
     driver_namespace = 'keystone.identity.shadow_users'
 
     def __init__(self):
-        super(ShadowUsersManager, self).__init__(CONF.shadow_users.driver)
+        shadow_driver = CONF.shadow_users.driver
+
+        super(ShadowUsersManager, self).__init__(shadow_driver)
+
+        if isinstance(self.driver, shadow_interface.ShadowUsersDriverV9):
+            self.driver = (
+                shadow_interface.V10ShadowUsersWrapperForV9Driver(self.driver))
+        elif not isinstance(self.driver,
+                            shadow_interface.ShadowUsersDriverV10):
+            raise exception.UnsupportedDriverVersion(driver=shadow_driver)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ShadowUsersDriverV9(object):
-    """Interface description for an Shadow Users driver."""
-
-    @abc.abstractmethod
-    def create_federated_user(self, federated_dict):
-        """Create a new user with the federated identity
-
-        :param dict federated_dict: Reference to the federated user
-        :param user_id: user ID for linking to the federated identity
-        :returns dict: Containing the user reference
-
-        """
-        raise exception.NotImplemented()
-
-    @abc.abstractmethod
-    def get_federated_user(self, idp_id, protocol_id, unique_id):
-        """Returns the found user for the federated identity
-
-        :param idp_id: The identity provider ID
-        :param protocol_id: The federation protocol ID
-        :param unique_id: The unique ID for the user
-        :returns dict: Containing the user reference
-
-        """
-        raise exception.NotImplemented()
-
-    @abc.abstractmethod
-    def update_federated_user_display_name(self, idp_id, protocol_id,
-                                           unique_id, display_name):
-        """Updates federated user's display name if changed
-
-        :param idp_id: The identity provider ID
-        :param protocol_id: The federation protocol ID
-        :param unique_id: The unique ID for the user
-        :param display_name: The user's display name
-
-        """
-        raise exception.NotImplemented()
+@versionutils.deprecated(
+    versionutils.deprecated.NEWTON,
+    what='keystone.identity.ShadowUsersDriverV9',
+    in_favor_of='keystone.identity.shadow_backends.base.ShadowUsersDriverV9',
+    remove_in=+1)
+class ShadowUsersDriverV9(shadow_interface.ShadowUsersDriverV9):
+    pass
