@@ -12,15 +12,13 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-"""Notifications module for OpenStack Identity Service resources"""
+"""Notifications module for OpenStack Identity Service resources."""
 
 import collections
 import functools
 import inspect
-import logging
 import socket
 
-from oslo_config import cfg
 from oslo_log import log
 import oslo_messaging
 from oslo_utils import reflection
@@ -32,31 +30,11 @@ from pycadf import eventfactory
 from pycadf import resource
 
 from keystone.i18n import _, _LE
+from keystone.common import dependency
 from keystone.common import utils
+import keystone.conf
 
-
-notifier_opts = [
-    cfg.StrOpt('default_publisher_id',
-               help='Default publisher_id for outgoing notifications'),
-    cfg.StrOpt('notification_format', default='basic',
-               choices=['basic', 'cadf'],
-               help='Define the notification format for Identity Service '
-                    'events. A "basic" notification has information about '
-                    'the resource being operated on. A "cadf" notification '
-                    'has the same information, as well as information about '
-                    'the initiator of the event.'),
-    cfg.MultiStrOpt('notification_opt_out', default=[],
-                    help='Define the notification options to opt-out from. '
-                         'The value expected is: '
-                         'identity.<resource_type>.<operation>. This field '
-                         'can be set multiple times in order to add more '
-                         'notifications to opt-out from. For example:\n '
-                         'notification_opt_out=identity.user.created\n '
-                         'notification_opt_out=identity.authenticate.success'),
-]
-
-config_section = None
-list_opts = lambda: [(config_section, notifier_opts), ]
+_CATALOG_HELPER_OBJ = None
 
 LOG = log.getLogger(__name__)
 # NOTE(gyee): actions that can be notified. One must update this list whenever
@@ -91,8 +69,7 @@ _notifier = None
 SERVICE = 'identity'
 
 
-CONF = cfg.CONF
-CONF.register_opts(notifier_opts)
+CONF = keystone.conf.CONF
 
 # NOTE(morganfainberg): Special case notifications that are only used
 # internally for handling token persistence token deletions
@@ -244,7 +221,7 @@ def register_event_callback(event, resource_type, callbacks):
         _SUBSCRIBERS.setdefault(event, {}).setdefault(resource_type, set())
         _SUBSCRIBERS[event][resource_type].add(callback)
 
-        if LOG.logger.getEffectiveLevel() <= logging.DEBUG:
+        if LOG.logger.getEffectiveLevel() <= log.DEBUG:
             # Do this only if its going to appear in the logs.
             msg = 'Callback: `%(callback)s` subscribed to event `%(event)s`.'
             callback_info = _get_callback_info(callback)
@@ -301,7 +278,7 @@ def listener(cls):
 
 
 def notify_event_callbacks(service, resource_type, operation, payload):
-    """Sends a notification to registered extensions."""
+    """Send a notification to registered extensions."""
     if operation in _SUBSCRIBERS:
         if resource_type in _SUBSCRIBERS[operation]:
             for cb in _SUBSCRIBERS[operation][resource_type]:
@@ -329,7 +306,7 @@ def _get_notifier():
     if _notifier is None:
         host = CONF.default_publisher_id or socket.gethostname()
         try:
-            transport = oslo_messaging.get_transport(CONF)
+            transport = oslo_messaging.get_notification_transport(CONF)
             _notifier = oslo_messaging.Notifier(transport,
                                                 "identity.%s" % host)
         except Exception:
@@ -503,12 +480,12 @@ class CadfNotificationWrapper(object):
 
     def __call__(self, f):
         @functools.wraps(f)
-        def wrapper(wrapped_self, context, user_id, *args, **kwargs):
-            """Always send a notification."""
-            initiator = _get_request_audit_info(context, user_id)
+        def wrapper(wrapped_self, request, user_id, *args, **kwargs):
+            """Alway send a notification."""
+            initiator = _get_request_audit_info(request.context_dict, user_id)
             target = resource.Resource(typeURI=taxonomy.ACCOUNT_USER)
             try:
-                result = f(wrapped_self, context, user_id, *args, **kwargs)
+                result = f(wrapped_self, request, user_id, *args, **kwargs)
             except Exception:
                 # For authentication failure send a cadf event as well
                 _send_audit_notification(self.action, initiator,
@@ -662,6 +639,11 @@ def send_saml_audit_notification(action, context, user_id, group_ids,
     _send_audit_notification(action, initiator, outcome, target, event_type)
 
 
+@dependency.requires('catalog_api')
+class _CatalogHelperObj(object):
+    """A helper object to allow lookups of identity service id."""
+
+
 def _send_audit_notification(action, initiator, outcome, target,
                              event_type, **kwargs):
     """Send CADF notification to inform observers about the affected resource.
@@ -682,6 +664,17 @@ def _send_audit_notification(action, initiator, outcome, target,
     if _check_notification_opt_out(event_type, outcome):
         return
 
+    global _CATALOG_HELPER_OBJ
+    if _CATALOG_HELPER_OBJ is None:
+        _CATALOG_HELPER_OBJ = _CatalogHelperObj()
+    service_list = _CATALOG_HELPER_OBJ.catalog_api.list_services()
+    service_id = None
+
+    for i in service_list:
+        if i['type'] == SERVICE:
+            service_id = i['id']
+            break
+
     event = eventfactory.EventFactory().new_event(
         eventType=cadftype.EVENTTYPE_ACTIVITY,
         outcome=outcome,
@@ -690,6 +683,9 @@ def _send_audit_notification(action, initiator, outcome, target,
         target=target,
         observer=resource.Resource(typeURI=taxonomy.SERVICE_SECURITY))
 
+    if service_id is not None:
+        event.observer.id = service_id
+
     for key, value in kwargs.items():
         setattr(event, key, value)
 
