@@ -32,14 +32,14 @@ class TestModelDictMixin(unit.BaseTestCase):
     def test_creating_a_model_instance_from_a_dict(self):
         d = {'id': utils.new_uuid(), 'text': utils.new_uuid()}
         m = TestModel.from_dict(d)
-        self.assertEqual(m.id, d['id'])
-        self.assertEqual(m.text, d['text'])
+        self.assertEqual(d['id'], m.id)
+        self.assertEqual(d['text'], m.text)
 
     def test_creating_a_dict_from_a_model_instance(self):
         m = TestModel(id=utils.new_uuid(), text=utils.new_uuid())
         d = m.to_dict()
-        self.assertEqual(m.id, d['id'])
-        self.assertEqual(m.text, d['text'])
+        self.assertEqual(d['id'], m.id)
+        self.assertEqual(d['text'], m.text)
 
     def test_creating_a_model_instance_from_an_invalid_dict(self):
         d = {'id': utils.new_uuid(), 'text': utils.new_uuid(), 'extra': None}
@@ -49,4 +49,4 @@ class TestModelDictMixin(unit.BaseTestCase):
         expected = {'id': utils.new_uuid(), 'text': utils.new_uuid()}
         m = TestModel(id=expected['id'], text=expected['text'])
         m.extra = 'this should not be in the dictionary'
-        self.assertEqual(m.to_dict(), expected)
+        self.assertEqual(expected, m.to_dict())
