@@ -17,7 +17,6 @@ import functools
 import uuid
 
 import mock
-from oslo_config import cfg
 from oslo_db import exception as db_exception
 from oslo_db import options
 from six.moves import range
@@ -27,14 +26,17 @@ from testtools import matchers
 
 from keystone.common import driver_hints
 from keystone.common import sql
+import keystone.conf
+from keystone.credential.providers import fernet as credential_provider
 from keystone import exception
-from keystone.identity.backends import sql as identity_sql
-from keystone import resource
+from keystone.identity.backends import sql_model as identity_sql
+from keystone.resource.backends import base as resource
 from keystone.tests import unit
 from keystone.tests.unit.assignment import test_backends as assignment_tests
 from keystone.tests.unit.catalog import test_backends as catalog_tests
 from keystone.tests.unit import default_fixtures
 from keystone.tests.unit.identity import test_backends as identity_tests
+from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import database
 from keystone.tests.unit.policy import test_backends as policy_tests
 from keystone.tests.unit.resource import test_backends as resource_tests
@@ -43,7 +45,7 @@ from keystone.tests.unit.trust import test_backends as trust_tests
 from keystone.token.persistence.backends import sql as token_sql
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 class SqlTests(unit.SQLDriverOverrides, unit.TestCase):
@@ -132,20 +134,27 @@ class SqlModels(SqlTests):
         cols = (('id', sql.String, 64),
                 ('default_project_id', sql.String, 64),
                 ('enabled', sql.Boolean, None),
-                ('extra', sql.JsonBlob, None))
+                ('extra', sql.JsonBlob, None),
+                ('created_at', sql.DateTime, None),
+                ('last_active_at', sqlalchemy.Date, None))
         self.assertExpectedSchema('user', cols)
 
     def test_local_user_model(self):
         cols = (('id', sql.Integer, None),
                 ('user_id', sql.String, 64),
                 ('name', sql.String, 255),
-                ('domain_id', sql.String, 64))
+                ('domain_id', sql.String, 64),
+                ('failed_auth_count', sql.Integer, None),
+                ('failed_auth_at', sql.DateTime, None))
         self.assertExpectedSchema('local_user', cols)
 
     def test_password_model(self):
         cols = (('id', sql.Integer, None),
                 ('local_user_id', sql.Integer, None),
-                ('password', sql.String, 128))
+                ('password', sql.String, 128),
+                ('created_at', sql.DateTime, None),
+                ('expires_at', sql.DateTime, None),
+                ('self_service', sql.Boolean, False))
         self.assertExpectedSchema('password', cols)
 
     def test_federated_user_model(self):
@@ -157,6 +166,12 @@ class SqlModels(SqlTests):
                 ('display_name', sql.String, 255))
         self.assertExpectedSchema('federated_user', cols)
 
+    def test_nonlocal_user_model(self):
+        cols = (('domain_id', sql.String, 64),
+                ('name', sql.String, 255),
+                ('user_id', sql.String, 64))
+        self.assertExpectedSchema('nonlocal_user', cols)
+
     def test_group_model(self):
         cols = (('id', sql.String, 64),
                 ('name', sql.String, 64),
@@ -165,13 +180,6 @@ class SqlModels(SqlTests):
                 ('extra', sql.JsonBlob, None))
         self.assertExpectedSchema('group', cols)
 
-    def test_domain_model(self):
-        cols = (('id', sql.String, 64),
-                ('name', sql.String, 64),
-                ('enabled', sql.Boolean, True),
-                ('extra', sql.JsonBlob, None))
-        self.assertExpectedSchema('domain', cols)
-
     def test_project_model(self):
         cols = (('id', sql.String, 64),
                 ('name', sql.String, 64),
@@ -213,7 +221,9 @@ class SqlModels(SqlTests):
         self.assertExpectedSchema('revocation_event', cols)
 
 
-class SqlIdentity(SqlTests, identity_tests.IdentityTests,
+class SqlIdentity(SqlTests,
+                  identity_tests.IdentityTests,
+                  identity_tests.ShadowUsersTests,
                   assignment_tests.AssignmentTests,
                   resource_tests.ResourceTests):
     def test_password_hashed(self):
@@ -231,7 +241,7 @@ class SqlIdentity(SqlTests, identity_tests.IdentityTests,
         with sql.session_for_read() as session:
             new_user_ref = self.identity_api._get_user(session,
                                                        new_user_dict['id'])
-            self.assertFalse(new_user_ref.local_user.passwords)
+            self.assertIsNone(new_user_ref.password)
 
     def test_update_user_with_null_password(self):
         user_dict = unit.new_user_ref(
@@ -244,7 +254,7 @@ class SqlIdentity(SqlTests, identity_tests.IdentityTests,
         with sql.session_for_read() as session:
             new_user_ref = self.identity_api._get_user(session,
                                                        new_user_dict['id'])
-            self.assertFalse(new_user_ref.local_user.passwords)
+            self.assertIsNone(new_user_ref.password)
 
     def test_delete_user_with_project_association(self):
         user = unit.new_user_ref(domain_id=CONF.identity.default_domain_id)
@@ -281,44 +291,6 @@ class SqlIdentity(SqlTests, identity_tests.IdentityTests,
         ref['name'] = ref['name'].upper()
         self.identity_api.create_user(ref)
 
-    def test_create_federated_user_unique_constraint(self):
-        federated_dict = unit.new_federated_user_ref()
-        user_dict = self.shadow_users_api.create_federated_user(federated_dict)
-        user_dict = self.identity_api.get_user(user_dict["id"])
-        self.assertIsNotNone(user_dict["id"])
-        self.assertRaises(exception.Conflict,
-                          self.shadow_users_api.create_federated_user,
-                          federated_dict)
-
-    def test_get_federated_user(self):
-        federated_dict = unit.new_federated_user_ref()
-        user_dict_create = self.shadow_users_api.create_federated_user(
-            federated_dict)
-        user_dict_get = self.shadow_users_api.get_federated_user(
-            federated_dict["idp_id"],
-            federated_dict["protocol_id"],
-            federated_dict["unique_id"])
-        self.assertItemsEqual(user_dict_create, user_dict_get)
-        self.assertEqual(user_dict_create["id"], user_dict_get["id"])
-
-    def test_update_federated_user_display_name(self):
-        federated_dict = unit.new_federated_user_ref()
-        user_dict_create = self.shadow_users_api.create_federated_user(
-            federated_dict)
-        new_display_name = uuid.uuid4().hex
-        self.shadow_users_api.update_federated_user_display_name(
-            federated_dict["idp_id"],
-            federated_dict["protocol_id"],
-            federated_dict["unique_id"],
-            new_display_name)
-        user_ref = self.shadow_users_api._get_federated_user(
-            federated_dict["idp_id"],
-            federated_dict["protocol_id"],
-            federated_dict["unique_id"])
-        self.assertEqual(user_ref.federated_users[0].display_name,
-                         new_display_name)
-        self.assertEqual(user_dict_create["id"], user_ref.id)
-
     def test_create_project_case_sensitivity(self):
         # project name case sensitivity is down to the fact that it is marked
         # as an SQL UNIQUE column, which may not be valid for other backends,
@@ -333,21 +305,6 @@ class SqlIdentity(SqlTests, identity_tests.IdentityTests,
         ref['name'] = ref['name'].upper()
         self.resource_api.create_project(ref['id'], ref)
 
-    def test_create_null_project_name(self):
-        project = unit.new_project_ref(
-            name=None, domain_id=CONF.identity.default_domain_id)
-        self.assertRaises(exception.ValidationError,
-                          self.resource_api.create_project,
-                          project['id'],
-                          project)
-        self.assertRaises(exception.ProjectNotFound,
-                          self.resource_api.get_project,
-                          project['id'])
-        self.assertRaises(exception.ProjectNotFound,
-                          self.resource_api.get_project_by_name,
-                          project['name'],
-                          CONF.identity.default_domain_id)
-
     def test_delete_project_with_user_association(self):
         user = unit.new_user_ref(domain_id=CONF.identity.default_domain_id)
         user = self.identity_api.create_user(user)
@@ -358,7 +315,7 @@ class SqlIdentity(SqlTests, identity_tests.IdentityTests,
         self.assertEqual([], tenants)
 
     def test_update_project_returns_extra(self):
-        """This tests for backwards-compatibility with an essex/folsom bug.
+        """Test for backward compatibility with an essex/folsom bug.
 
         Non-indexed attributes were returned in an 'extra' attribute, instead
         of on the entity itself; for consistency and backwards compatibility,
@@ -382,7 +339,7 @@ class SqlIdentity(SqlTests, identity_tests.IdentityTests,
         self.assertEqual(arbitrary_value, ref['extra'][arbitrary_key])
 
     def test_update_user_returns_extra(self):
-        """This tests for backwards-compatibility with an essex/folsom bug.
+        """Test for backwards-compatibility with an essex/folsom bug.
 
         Non-indexed attributes were returned in an 'extra' attribute, instead
         of on the entity itself; for consistency and backwards compatibility,
@@ -635,6 +592,43 @@ class SqlIdentity(SqlTests, identity_tests.IdentityTests,
         _exercise_project_api(uuid.uuid4().hex)
         _exercise_project_api(resource.NULL_DOMAIN_ID)
 
+    def test_list_users_call_count(self):
+        """There should not be O(N) queries."""
+        # create 10 users. 10 is just a random number
+        for i in range(10):
+            user = unit.new_user_ref(domain_id=CONF.identity.default_domain_id)
+            self.identity_api.create_user(user)
+
+        # sqlalchemy emits various events and allows to listen to them. Here
+        # bound method `query_counter` will be called each time when a query
+        # is compiled
+        class CallCounter(object):
+            def __init__(self):
+                self.calls = 0
+
+            def reset(self):
+                self.calls = 0
+
+            def query_counter(self, query):
+                self.calls += 1
+
+        counter = CallCounter()
+        sqlalchemy.event.listen(sqlalchemy.orm.query.Query, 'before_compile',
+                                counter.query_counter)
+
+        first_call_users = self.identity_api.list_users()
+        first_call_counter = counter.calls
+        # add 10 more users
+        for i in range(10):
+            user = unit.new_user_ref(domain_id=CONF.identity.default_domain_id)
+            self.identity_api.create_user(user)
+        counter.reset()
+        second_call_users = self.identity_api.list_users()
+        # ensure that the number of calls does not depend on the number of
+        # users fetched.
+        self.assertNotEqual(len(first_call_users), len(second_call_users))
+        self.assertEqual(first_call_counter, counter.calls)
+
 
 class SqlTrust(SqlTests, trust_tests.TrustTests):
     pass
@@ -846,11 +840,50 @@ class SqlImpliedRoles(SqlTests, assignment_tests.ImpliedRoleTests):
     pass
 
 
-class SqlTokenCacheInvalidation(SqlTests, token_tests.TokenCacheInvalidation):
+class SqlTokenCacheInvalidationWithUUID(SqlTests,
+                                        token_tests.TokenCacheInvalidation):
     def setUp(self):
-        super(SqlTokenCacheInvalidation, self).setUp()
+        super(SqlTokenCacheInvalidationWithUUID, self).setUp()
         self._create_test_data()
 
+    def config_overrides(self):
+        super(SqlTokenCacheInvalidationWithUUID, self).config_overrides()
+        # NOTE(lbragstad): The TokenCacheInvalidation tests are coded to work
+        # against a persistent token backend. Only run these with token
+        # providers that issue persistent tokens.
+        self.config_fixture.config(group='token', provider='uuid')
+
+
+class SqlTokenCacheInvalidationWithPKI(SqlTests,
+                                       token_tests.TokenCacheInvalidation):
+    def setUp(self):
+        super(SqlTokenCacheInvalidationWithPKI, self).setUp()
+        self._create_test_data()
+
+    def config_overrides(self):
+        super(SqlTokenCacheInvalidationWithPKI, self).config_overrides()
+        # NOTE(lbragstad): The TokenCacheInvalidation tests are coded to work
+        # against a persistent token backend. Only run these with token
+        # providers that issue persistent tokens.
+        self.config_fixture.config(group='token', provider='pki')
+
+
+class SqlTokenCacheInvalidationWithPKIZ(SqlTests,
+                                        token_tests.TokenCacheInvalidation):
+    def setUp(self):
+        super(SqlTokenCacheInvalidationWithPKIZ, self).setUp()
+        self._create_test_data()
+
+    def config_overrides(self):
+        super(SqlTokenCacheInvalidationWithPKIZ, self).config_overrides()
+        # NOTE(lbragstad): The TokenCacheInvalidation tests are coded to work
+        # against a persistent token backend. Only run these with token
+        # providers that issue persistent tokens.
+        self.config_fixture.config(group='token', provider='pkiz')
+
+# NOTE(lbragstad): The Fernet token provider doesn't persist tokens in a
+# backend, so running the TokenCacheInvalidation tests here doesn't make sense.
+
 
 class SqlFilterTests(SqlTests, identity_tests.FilterTests):
 
@@ -894,7 +927,7 @@ class SqlFilterTests(SqlTests, identity_tests.FilterTests):
             self.assertFalse(hints.get_exact_filter_by_name('domain_id'))
 
     def test_filter_sql_injection_attack(self):
-        """Test against sql injection attack on filters
+        """Test against sql injection attack on filters.
 
         Test Plan:
         - Attempt to get all entities back by passing a two-term attribute
@@ -903,7 +936,7 @@ class SqlFilterTests(SqlTests, identity_tests.FilterTests):
         """
         # Check we have some users
         users = self.identity_api.list_users()
-        self.assertTrue(len(users) > 0)
+        self.assertGreater(len(users), 0)
 
         hints = driver_hints.Hints()
         hints.add_filter('name', "anything' or 'x'='x")
@@ -921,7 +954,7 @@ class SqlFilterTests(SqlTests, identity_tests.FilterTests):
         self.assertEqual(0, len(groups))
 
         groups = self.identity_api.list_groups()
-        self.assertTrue(len(groups) > 0)
+        self.assertGreater(len(groups), 0)
 
 
 class SqlLimitTests(SqlTests, identity_tests.LimitTests):
@@ -993,7 +1026,16 @@ class SqlCredential(SqlTests):
             self.assertIn(cred['id'], retrived_ids)
 
     def setUp(self):
+        self.useFixture(database.Database())
         super(SqlCredential, self).setUp()
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'credential',
+                credential_provider.MAX_ACTIVE_KEYS
+            )
+        )
+
         self.credentials = []
         for _ in range(3):
             self.credentials.append(
@@ -1023,3 +1065,49 @@ class SqlCredential(SqlTests):
         credentials = self.credential_api.list_credentials_for_user(
             self.user_foo['id'], type=cred['type'])
         self._validateCredentialList(credentials, [cred])
+
+    def test_create_credential_is_encrypted_when_stored(self):
+        credential = unit.new_credential_ref(user_id=uuid.uuid4().hex)
+        credential_id = credential['id']
+        returned_credential = self.credential_api.create_credential(
+            credential_id,
+            credential
+        )
+
+        # Make sure the `blob` is *not* encrypted when returned from the
+        # credential API.
+        self.assertEqual(returned_credential['blob'], credential['blob'])
+
+        credential_from_backend = self.credential_api.driver.get_credential(
+            credential_id
+        )
+
+        # Pull the credential directly from the backend, the `blob` should be
+        # encrypted.
+        self.assertNotEqual(
+            credential_from_backend['encrypted_blob'],
+            credential['blob']
+        )
+
+    def test_list_credentials_is_decrypted(self):
+        credential = unit.new_credential_ref(user_id=uuid.uuid4().hex)
+        credential_id = credential['id']
+
+        created_credential = self.credential_api.create_credential(
+            credential_id,
+            credential
+        )
+
+        # Pull the credential directly from the backend, the `blob` should be
+        # encrypted.
+        credential_from_backend = self.credential_api.driver.get_credential(
+            credential_id
+        )
+        self.assertNotEqual(
+            credential_from_backend['encrypted_blob'],
+            credential['blob']
+        )
+
+        # Make sure the `blob` values listed from the API are not encrypted.
+        listed_credentials = self.credential_api.list_credentials()
+        self.assertIn(created_credential, listed_credentials)
