@@ -12,22 +12,26 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import logging
+import datetime
 import uuid
 
 import fixtures
+import freezegun
 import mock
-from oslo_config import cfg
+from oslo_log import log
 from six.moves import http_client
 from testtools import matchers
 
 from keystone.common import controller
+import keystone.conf
+from keystone.credential.providers import fernet as credential_fernet
 from keystone import exception
 from keystone.tests import unit
+from keystone.tests.unit import ksfixtures
 from keystone.tests.unit import test_v3
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 # NOTE(morganfainberg): To be removed when admin_token_auth middleware is
@@ -70,6 +74,13 @@ class IdentityTestCase(test_v3.RestfulTestCase):
 
     def setUp(self):
         super(IdentityTestCase, self).setUp()
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'credential',
+                credential_fernet.MAX_ACTIVE_KEYS
+            )
+        )
 
         self.group = unit.new_group_ref(domain_id=self.domain_id)
         self.group = self.identity_api.create_group(self.group)
@@ -228,12 +239,13 @@ class IdentityTestCase(test_v3.RestfulTestCase):
                         "DEFaUlT"}},
                   expected_status=http_client.NOT_FOUND)
 
-    def test_list_users(self):
-        """Call ``GET /users``."""
+    def test_list_head_users(self):
+        """Call ``GET & HEAD /users``."""
         resource_url = '/users'
         r = self.get(resource_url)
         self.assertValidUserListResponse(r, ref=self.user,
                                          resource_url=resource_url)
+        self.head(resource_url, expected_status=http_client.OK)
 
     def test_list_users_with_multiple_backends(self):
         """Call ``GET /users`` when multiple backends is enabled.
@@ -300,11 +312,33 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         self.assertValidUserListResponse(r, ref=user,
                                          resource_url=resource_url)
 
-    def test_get_user(self):
-        """Call ``GET /users/{user_id}``."""
-        r = self.get('/users/%(user_id)s' % {
-            'user_id': self.user['id']})
+    def test_get_head_user(self):
+        """Call ``GET & HEAD /users/{user_id}``."""
+        resource_url = '/users/%(user_id)s' % {
+            'user_id': self.user['id']}
+        r = self.get(resource_url)
         self.assertValidUserResponse(r, self.user)
+        self.head(resource_url, expected_status=http_client.OK)
+
+    def test_get_user_does_not_include_extra_attributes(self):
+        """Call ``GET /users/{user_id}`` extra attributes are not included."""
+        user = unit.new_user_ref(domain_id=self.domain_id,
+                                 project_id=self.project_id)
+        user = self.identity_api.create_user(user)
+        self.assertNotIn('created_at', user)
+        self.assertNotIn('last_active_at', user)
+
+    def test_get_user_includes_required_attributes(self):
+        """Call ``GET /users/{user_id}`` required attributes are included."""
+        user = unit.new_user_ref(domain_id=self.domain_id,
+                                 project_id=self.project_id)
+        user = self.identity_api.create_user(user)
+        self.assertIn('id', user)
+        self.assertIn('name', user)
+        self.assertIn('enabled', user)
+        self.assertIn('password_expires_at', user)
+        r = self.get('/users/%(user_id)s' % {'user_id': user['id']})
+        self.assertValidUserResponse(r, user)
 
     def test_get_user_with_default_project(self):
         """Call ``GET /users/{user_id}`` making sure of default_project_id."""
@@ -319,8 +353,8 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         self.put('/groups/%(group_id)s/users/%(user_id)s' % {
             'group_id': self.group_id, 'user_id': self.user['id']})
 
-    def test_list_groups_for_user(self):
-        """Call ``GET /users/{user_id}/groups``."""
+    def test_list_head_groups_for_user(self):
+        """Call ``GET & HEAD /users/{user_id}/groups``."""
         user1 = unit.create_user(self.identity_api,
                                  domain_id=self.domain['id'])
         user2 = unit.create_user(self.identity_api,
@@ -340,6 +374,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         r = self.get(resource_url, auth=auth)
         self.assertValidGroupListResponse(r, ref=self.group,
                                           resource_url=resource_url)
+        self.head(resource_url, auth=auth, expected_status=http_client.OK)
 
         # Administrator is allowed to list others' groups
         resource_url = ('/users/%(user_id)s/groups' %
@@ -347,14 +382,18 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         r = self.get(resource_url)
         self.assertValidGroupListResponse(r, ref=self.group,
                                           resource_url=resource_url)
+        self.head(resource_url, expected_status=http_client.OK)
 
         # Ordinary users should not be allowed to list other's groups
         auth = self.build_authentication_request(
             user_id=user2['id'],
             password=user2['password'])
-        r = self.get('/users/%(user_id)s/groups' % {
-            'user_id': user1['id']}, auth=auth,
-            expected_status=exception.ForbiddenAction.code)
+        resource_url = '/users/%(user_id)s/groups' % {
+            'user_id': user1['id']}
+        self.get(resource_url, auth=auth,
+                 expected_status=exception.ForbiddenAction.code)
+        self.head(resource_url, auth=auth,
+                  expected_status=exception.ForbiddenAction.code)
 
     def test_check_user_in_group(self):
         """Call ``HEAD /groups/{group_id}/users/{user_id}``."""
@@ -363,8 +402,8 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         self.head('/groups/%(group_id)s/users/%(user_id)s' % {
             'group_id': self.group_id, 'user_id': self.user['id']})
 
-    def test_list_users_in_group(self):
-        """Call ``GET /groups/{group_id}/users``."""
+    def test_list_head_users_in_group(self):
+        """Call ``GET & HEAD /groups/{group_id}/users``."""
         self.put('/groups/%(group_id)s/users/%(user_id)s' % {
             'group_id': self.group_id, 'user_id': self.user['id']})
         resource_url = ('/groups/%(group_id)s/users' %
@@ -374,6 +413,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
                                          resource_url=resource_url)
         self.assertIn('/groups/%(group_id)s/users' % {
             'group_id': self.group_id}, r.result['links']['self'])
+        self.head(resource_url, expected_status=http_client.OK)
 
     def test_remove_user_from_group(self):
         """Call ``DELETE /groups/{group_id}/users/{user_id}``."""
@@ -426,6 +466,25 @@ class IdentityTestCase(test_v3.RestfulTestCase):
             password=new_password)
         self.v3_create_token(new_password_auth)
 
+    def test_admin_password_reset_with_min_password_age_enabled(self):
+        # enable minimum_password_age, this should have no effect on admin
+        # password reset
+        self.config_fixture.config(group='security_compliance',
+                                   minimum_password_age=1)
+        # create user
+        user_ref = unit.create_user(self.identity_api,
+                                    domain_id=self.domain['id'])
+        # administrative password reset
+        new_password = uuid.uuid4().hex
+        r = self.patch('/users/%s' % user_ref['id'],
+                       body={'user': {'password': new_password}})
+        self.assertValidUserResponse(r, user_ref)
+        # authenticate with new password
+        new_password_auth = self.build_authentication_request(
+            user_id=user_ref['id'],
+            password=new_password)
+        self.v3_create_token(new_password_auth)
+
     def test_update_user_domain_id(self):
         """Call ``PATCH /users/{user_id}`` with domain_id."""
         user = unit.new_user_ref(domain_id=self.domain['id'])
@@ -492,48 +551,6 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         r = self.credential_api.get_credential(credential2['id'])
         self.assertDictEqual(credential2, r)
 
-    # shadow user tests
-    def test_shadow_federated_user(self):
-        fed_user = unit.new_federated_user_ref()
-        user = (
-            self.identity_api.shadow_federated_user(fed_user["idp_id"],
-                                                    fed_user["protocol_id"],
-                                                    fed_user["unique_id"],
-                                                    fed_user["display_name"])
-        )
-        self.assertIsNotNone(user["id"])
-        self.assertEqual(len(user.keys()), 4)
-        self.assertIsNotNone(user['id'])
-        self.assertIsNotNone(user['name'])
-        self.assertIsNone(user['domain_id'])
-        self.assertEqual(user['enabled'], True)
-
-    def test_shadow_existing_federated_user(self):
-        fed_user = unit.new_federated_user_ref()
-
-        # introduce the user to keystone for the first time
-        shadow_user1 = self.identity_api.shadow_federated_user(
-            fed_user["idp_id"],
-            fed_user["protocol_id"],
-            fed_user["unique_id"],
-            fed_user["display_name"])
-        self.assertEqual(fed_user['display_name'], shadow_user1['name'])
-
-        # shadow the user again, with another name to invalidate the cache
-        # internally, this operation causes request to the driver. It should
-        # not fail.
-        fed_user['display_name'] = uuid.uuid4().hex
-        shadow_user2 = self.identity_api.shadow_federated_user(
-            fed_user["idp_id"],
-            fed_user["protocol_id"],
-            fed_user["unique_id"],
-            fed_user["display_name"])
-        self.assertEqual(fed_user['display_name'], shadow_user2['name'])
-        self.assertNotEqual(shadow_user1['name'], shadow_user2['name'])
-
-        # The shadowed users still share the same unique ID.
-        self.assertEqual(shadow_user1['id'], shadow_user2['id'])
-
     # group crud tests
 
     def test_create_group(self):
@@ -550,18 +567,21 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         self.post('/groups', body={'group': {}},
                   expected_status=http_client.BAD_REQUEST)
 
-    def test_list_groups(self):
-        """Call ``GET /groups``."""
+    def test_list_head_groups(self):
+        """Call ``GET & HEAD /groups``."""
         resource_url = '/groups'
         r = self.get(resource_url)
         self.assertValidGroupListResponse(r, ref=self.group,
                                           resource_url=resource_url)
+        self.head(resource_url, expected_status=http_client.OK)
 
-    def test_get_group(self):
-        """Call ``GET /groups/{group_id}``."""
-        r = self.get('/groups/%(group_id)s' % {
-            'group_id': self.group_id})
+    def test_get_head_group(self):
+        """Call ``GET & HEAD /groups/{group_id}``."""
+        resource_url = '/groups/%(group_id)s' % {
+            'group_id': self.group_id}
+        r = self.get(resource_url)
         self.assertValidGroupResponse(r, self.group)
+        self.head(resource_url, expected_status=http_client.OK)
 
     def test_update_group(self):
         """Call ``PATCH /groups/{group_id}``."""
@@ -594,7 +614,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
     def test_create_user_password_not_logged(self):
         # When a user is created, the password isn't logged at any level.
 
-        log_fix = self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
+        log_fix = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
 
         ref = unit.new_user_ref(domain_id=self.domain_id)
         self.post(
@@ -607,7 +627,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         # When admin modifies user password, the password isn't logged at any
         # level.
 
-        log_fix = self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
+        log_fix = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
 
         # bootstrap a user as admin
         user_ref = unit.create_user(self.identity_api,
@@ -622,6 +642,32 @@ class IdentityTestCase(test_v3.RestfulTestCase):
 
         self.assertNotIn(new_password, log_fix.output)
 
+    def test_setting_default_project_id_to_domain_failed(self):
+        """Call ``POST and PATCH /users`` default_project_id=domain_id.
+
+        Make sure we validate the default_project_id if it is specified.
+        It cannot be set to a domain_id, even for a project acting as domain
+        right now. That's because we haven't sort out the issuing
+        project-scoped token for project acting as domain bit yet. Once we
+        got that sorted out, we can relax this constraint.
+
+        """
+        # creating a new user with default_project_id set to a
+        # domain_id should result in HTTP 400
+        ref = unit.new_user_ref(domain_id=self.domain_id,
+                                project_id=self.domain_id)
+        self.post('/users', body={'user': ref}, token=CONF.admin_token,
+                  expected_status=http_client.BAD_REQUEST)
+
+        # updating user's default_project_id to a domain_id should result
+        # in HTTP 400
+        user = {'default_project_id': self.domain_id}
+        self.patch('/users/%(user_id)s' % {
+            'user_id': self.user['id']},
+            body={'user': user},
+            token=CONF.admin_token,
+            expected_status=http_client.BAD_REQUEST)
+
 
 class IdentityV3toV2MethodsTestCase(unit.TestCase):
     """Test users V3 to V2 conversion methods."""
@@ -668,6 +714,13 @@ class IdentityV3toV2MethodsTestCase(unit.TestCase):
             name=user_id,
             tenantId=project_id,
             domain_id=CONF.identity.default_domain_id)
+        # User with password_expires_at
+        self.user5 = self.new_user_ref(
+            id=user_id,
+            name=user_id,
+            project_id=project_id,
+            domain_id=CONF.identity.default_domain_id,
+            password_expires_at=None)
 
         # Expected result if the user is meant to have a tenantId element
         self.expected_user = {'id': user_id,
@@ -694,9 +747,16 @@ class IdentityV3toV2MethodsTestCase(unit.TestCase):
         updated_user4 = controller.V2Controller.v3_to_v2_user(self.user4)
         self.assertIs(self.user4, updated_user4)
         self.assertDictEqual(self.expected_user_no_tenant_id, self.user4)
+        # password_expires_at filter test
+        password_expires_at_key = 'password_expires_at'
+        self.assertIn(password_expires_at_key, self.user5)
+        updated_user5 = controller.V2Controller.v3_to_v2_user(self.user5)
+        self.assertIs(self.user5, updated_user5)
+        self.assertNotIn(password_expires_at_key, updated_user5)
 
     def test_v3_to_v2_user_method_list(self):
-        user_list = [self.user1, self.user2, self.user3, self.user4]
+        user_list = [self.user1, self.user2, self.user3, self.user4,
+                     self.user5]
         updated_list = controller.V2Controller.v3_to_v2_user(user_list)
 
         self.assertEqual(len(user_list), len(updated_list))
@@ -704,6 +764,7 @@ class IdentityV3toV2MethodsTestCase(unit.TestCase):
         for i, ref in enumerate(updated_list):
             # Order should not change.
             self.assertIs(ref, user_list[i])
+            self.assertNotIn('password_expires_at', user_list[i])
 
         self.assertDictEqual(self.expected_user, self.user1)
         self.assertDictEqual(self.expected_user_no_tenant_id, self.user2)
@@ -729,7 +790,7 @@ class UserSelfServiceChangingPasswordsTestCase(test_v3.RestfulTestCase):
         return r.headers.get('X-Subject-Token')
 
     def change_password(self, expected_status, **kwargs):
-        """Returns a test response for a change password request."""
+        """Return a test response for a change password request."""
         return self.post('/users/%s/password' % self.user_ref['id'],
                          body={'user': kwargs},
                          token=self.token,
@@ -761,6 +822,34 @@ class UserSelfServiceChangingPasswordsTestCase(test_v3.RestfulTestCase):
         self.get_request_token(new_password,
                                expected_status=http_client.CREATED)
 
+    def test_changing_password_with_min_password_age(self):
+        time = datetime.datetime.utcnow()
+        with freezegun.freeze_time(time) as frozen_datetime:
+            # enable minimum_password_age and attempt to change password
+            new_password = uuid.uuid4().hex
+            self.config_fixture.config(group='security_compliance',
+                                       minimum_password_age=1)
+            # able to change password after create user
+            self.change_password(password=new_password,
+                                 original_password=self.user_ref['password'],
+                                 expected_status=http_client.NO_CONTENT)
+            # 2nd change password should fail due to minimum password age and
+            # make sure we wait one second to avoid race conditions with Fernet
+            frozen_datetime.tick(delta=datetime.timedelta(seconds=1))
+            self.token = self.get_request_token(
+                new_password,
+                http_client.CREATED
+            )
+            self.change_password(password=uuid.uuid4().hex,
+                                 original_password=new_password,
+                                 expected_status=http_client.BAD_REQUEST)
+            # disable minimum_password_age and attempt to change password
+            self.config_fixture.config(group='security_compliance',
+                                       minimum_password_age=0)
+            self.change_password(password=uuid.uuid4().hex,
+                                 original_password=new_password,
+                                 expected_status=http_client.NO_CONTENT)
+
     def test_changing_password_with_missing_original_password_fails(self):
         r = self.change_password(password=uuid.uuid4().hex,
                                  expected_status=http_client.BAD_REQUEST)
@@ -792,7 +881,7 @@ class UserSelfServiceChangingPasswordsTestCase(test_v3.RestfulTestCase):
         # When a user changes their password, the password isn't logged at any
         # level.
 
-        log_fix = self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
+        log_fix = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
 
         # change password
         new_password = uuid.uuid4().hex
@@ -802,3 +891,58 @@ class UserSelfServiceChangingPasswordsTestCase(test_v3.RestfulTestCase):
 
         self.assertNotIn(self.user_ref['password'], log_fix.output)
         self.assertNotIn(new_password, log_fix.output)
+
+
+class PasswordValidationTestCase(UserSelfServiceChangingPasswordsTestCase):
+    """Test password validation."""
+
+    def setUp(self):
+        super(PasswordValidationTestCase, self).setUp()
+        # passwords requires: 1 letter, 1 digit, 7 chars
+        self.config_fixture.config(group='security_compliance',
+                                   password_regex=(
+                                       '^(?=.*\d)(?=.*[a-zA-Z]).{7,}$'))
+
+    def test_create_user_with_invalid_password(self):
+        user = unit.new_user_ref(domain_id=self.domain_id)
+        user['password'] = 'simple'
+        self.post('/users', body={'user': user}, token=self.get_admin_token(),
+                  expected_status=http_client.BAD_REQUEST)
+
+    def test_update_user_with_invalid_password(self):
+        user = unit.create_user(self.identity_api,
+                                domain_id=self.domain['id'])
+        user['password'] = 'simple'
+        self.patch('/users/%(user_id)s' % {
+            'user_id': user['id']},
+            body={'user': user},
+            expected_status=http_client.BAD_REQUEST)
+
+    def test_changing_password_with_simple_password_strength(self):
+        # password requires: any non-whitespace character
+        self.config_fixture.config(group='security_compliance',
+                                   password_regex='[\S]+')
+        self.change_password(password='simple',
+                             original_password=self.user_ref['password'],
+                             expected_status=http_client.NO_CONTENT)
+
+    def test_changing_password_with_strong_password_strength(self):
+        self.change_password(password='mypassword2',
+                             original_password=self.user_ref['password'],
+                             expected_status=http_client.NO_CONTENT)
+
+    def test_changing_password_with_strong_password_strength_fails(self):
+        # no digit
+        self.change_password(password='mypassword',
+                             original_password=self.user_ref['password'],
+                             expected_status=http_client.BAD_REQUEST)
+
+        # no letter
+        self.change_password(password='12345678',
+                             original_password=self.user_ref['password'],
+                             expected_status=http_client.BAD_REQUEST)
+
+        # less than 7 chars
+        self.change_password(password='mypas2',
+                             original_password=self.user_ref['password'],
+                             expected_status=http_client.BAD_REQUEST)
