@@ -12,22 +12,23 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""Keystone PKI Token Provider"""
+"""Keystone PKI Token Provider."""
+
+import subprocess  # nosec : used to catch subprocess exceptions
 
 from keystoneclient.common import cms
-from oslo_config import cfg
 from oslo_log import log
 from oslo_log import versionutils
 from oslo_serialization import jsonutils
 
-from keystone.common import environment
 from keystone.common import utils
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _, _LE
 from keystone.token.providers import common
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 LOG = log.getLogger(__name__)
 
@@ -48,7 +49,7 @@ class Provider(common.BaseProvider):
                                               CONF.signing.certfile,
                                               CONF.signing.keyfile))
             return token_id
-        except environment.subprocess.CalledProcessError:
+        except subprocess.CalledProcessError:
             LOG.exception(_LE('Unable to sign token'))
             raise exception.UnexpectedError(_(
                 'Unable to sign token.'))
