@@ -10,22 +10,23 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""Keystone Compressed PKI Token Provider"""
+"""Keystone Compressed PKI Token Provider."""
+
+import subprocess  # nosec : used to catch subprocess exceptions
 
 from keystoneclient.common import cms
-from oslo_config import cfg
 from oslo_log import log
 from oslo_log import versionutils
 from oslo_serialization import jsonutils
 
-from keystone.common import environment
 from keystone.common import utils
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 from keystone.token.providers import common
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 LOG = log.getLogger(__name__)
 ERROR_MESSAGE = _('Unable to sign token.')
@@ -47,7 +48,7 @@ class Provider(common.BaseProvider):
                                          CONF.signing.certfile,
                                          CONF.signing.keyfile))
             return token_id
-        except environment.subprocess.CalledProcessError:
+        except subprocess.CalledProcessError:
             LOG.exception(ERROR_MESSAGE)
             raise exception.UnexpectedError(ERROR_MESSAGE)
 
